@@ -3,8 +3,8 @@ from . import postprocess
 from . import preprocess
 from .utils import get_inheritors
 
-from .model.objects.element import Element
-from .model.optimization_setup import OptimizationSetup
+from .model.element import Element
+from .optimization_setup import OptimizationSetup
 
 # set the element classes of the EnergySystem class
 inheritors = get_inheritors(Element)
