@@ -18,8 +18,7 @@ from filelock import FileLock
 import yaml
 from pydantic import BaseModel
 
-from ..utils import HDFPandasSerializer
-from ..model.optimization_setup import OptimizationSetup
+from ..optimization_setup import OptimizationSetup
 
 
 # Warnings
@@ -88,37 +87,39 @@ class Postprocess:
         self.save_scenarios()
         self.save_solver()
         self.save_unit_definitions()
+        self.save_sequence_time_steps(scenario=scenario_name)
         self.save_param_map()
         if self.solver.run_diagnostics:
             self.save_benchmarking_data()
 
-        # extract and save sequence time steps, we transform the arrays to lists
-        self.dict_sequence_time_steps = self.flatten_dict(self.energy_system.time_steps.get_sequence_time_steps_dict())
-        self.dict_sequence_time_steps["optimized_time_steps"] = optimization_setup.optimized_time_steps
-        self.dict_sequence_time_steps["time_steps_operation_duration"] = self.energy_system.time_steps.time_steps_operation_duration
-        self.dict_sequence_time_steps["time_steps_storage_duration"] = self.energy_system.time_steps.time_steps_storage_duration
-        self.dict_sequence_time_steps["time_steps_storage_level_startend_year"] = self.energy_system.time_steps.time_steps_storage_level_startend_year
-        self.dict_sequence_time_steps["time_steps_year2operation"] = self.get_time_steps_year2operation()
-        self.dict_sequence_time_steps["time_steps_year2storage"] = self.get_time_steps_year2storage()
-
-        self.save_sequence_time_steps(scenario=scenario_name)
-
-    def write_file(self, name, dictionary, format=None):
+    def write_file(self, name, dictionary, format=None, mode = 'w'):
         """Writes the dictionary to file as json, if compression attribute is True, the serialized json is compressed
             and saved as binary file
 
         :param name: Filename without extension
         :param dictionary: The dictionary to save
         :param format: Force the format to use, if None use output_format attribute of instance
+        :param mode: Writting mode for python file. The two options are 'w' and
+            'a'. The former create a new file while the latter will append to an
+            existing file. Appending files is currently only supported for h5 
+            files. 
         """
 
         if isinstance(dictionary, BaseModel):
             dictionary = dictionary.model_dump()
 
+        # check whether valid mode
+        if not mode in ["a", "w"]:
+            ValueError(f"Invalid file write mode {mode} (valid options are 'a' or 'w').")
+
         # set the format
         if format is None:
             format = self.output_format
 
+        # only allow append mode for h5 files
+        if mode == 'a' and format != "h5":
+            raise ValueError(f"Write mode {mode} not available for output format {format}. If include_operation_only_phase = true, outputs must be saved in h5 files.")
+
         if format == "yml":
             # serialize to string
             serialized_dict = yaml.dump(dictionary)
@@ -151,7 +152,7 @@ class Postprocess:
         elif format == "h5":
             f_name = f"{name}.h5"
             with FileLock(f_name + ".lock").acquire(timeout=300):
-                HDFPandasSerializer.serialize_dict(file_name=f_name, dictionary=dictionary, overwrite=self.overwrite)
+                self._write_h5_file(f_name, dictionary, mode)
 
         elif format == "txt":
             f_name = f"{name}.txt"
@@ -175,10 +176,11 @@ class Postprocess:
         benchmarking_data["objective_value"] = self.model.objective.value
         if self.solver.name == "gurobi":
             benchmarking_data["solving_time"] = self.model.solver_model.Runtime
-            if self.solver.solver_options["Method"] == 2:
-                benchmarking_data["number_iterations"] = self.model.solver_model.BarIterCount
-            else:
-                benchmarking_data["number_iterations"] = self.model.solver_model.IterCount
+            if "Method" in self.solver.solver_options:
+                if self.solver.solver_options["Method"] == 2:
+                    benchmarking_data["number_iterations"] = self.model.solver_model.BarIterCount
+                else:
+                    benchmarking_data["number_iterations"] = self.model.solver_model.IterCount
             benchmarking_data["solver_status"] = self.model.solver_model.Status
             benchmarking_data["number_constraints"] = self.model.solver_model.NumConstrs
             benchmarking_data["number_variables"] = self.model.solver_model.NumVars
@@ -290,8 +292,20 @@ class Postprocess:
         # dataframe serialization
         data_frames = {}
         for name, arr in self.model.solution.items():
-            if self.solver.selected_saved_variables and name not in self.solver.selected_saved_variables:
+
+            # skip variables not selected to be saved
+            if (
+                not self.optimization_setup.operation_only_phase 
+                and self.solver.selected_saved_variables 
+                and name not in self.solver.selected_saved_variables
+            ) or (
+                self.optimization_setup.operation_only_phase
+                and self.solver.selected_saved_variables_operation
+                and name not in self.solver.selected_saved_variables_operation
+            ):
                 continue
+            
+            # extract doc information
             if name in self.vars.docs:
                 doc = self.vars.docs[name]
                 units = self.vars.units[name]
@@ -305,15 +319,32 @@ class Postprocess:
 
             # create dataframe
             df = arr.to_dataframe("value").dropna()
+
             # rename the index
             if len(df.index.names) == len(index_list):
                 df.index.names = index_list
 
             units = self._unit_df(units,df.index)
-            # we transform the dataframe to a json string and load it into the dictionary as dict
-            data_frames[name] = self._transform_df(df,doc,units)
 
-        self.write_file(self.name_dir.joinpath('var_dict'), data_frames)
+            # rename for operations-only duals
+            if self.optimization_setup.operation_only_phase:
+                name = name + '_operation'
+
+            # transform the dataframe to a json string and load it into the dictionary as dict
+            data_frames[name] = self._transform_df(df,doc,units)
+        
+        # choose whether to write new file or append to existing file
+        if self.optimization_setup.operation_only_phase:
+            mode = 'a'
+        else: 
+            mode = 'w'
+
+        # write file
+        self.write_file(
+            self.name_dir.joinpath('var_dict'), 
+            data_frames, 
+            mode = mode
+        )
 
     def save_duals(self):
         """ Saves the dual variable values to a json file which can then be
@@ -324,7 +355,23 @@ class Postprocess:
 
         # dataframe serialization
         data_frames = {}
-        for name, arr in self.model.dual.items():
+        for name in self.model.constraints:
+
+            arr = self.model.constraints[name].dual
+
+            # skip variables not selected to be saved
+            if (
+                not self.optimization_setup.operation_only_phase 
+                and self.solver.selected_saved_duals
+                and name not in self.solver.selected_saved_duals
+            ) or (
+                self.optimization_setup.operation_only_phase
+                and self.solver.selected_saved_duals_operation
+                and name not in self.solver.selected_saved_duals_operation
+            ):
+                continue
+            
+            # extract doc information
             if name in self.constraints.docs:
                 doc = self.constraints.docs[name]
                 index_list = self.get_index_list(doc)
@@ -332,6 +379,11 @@ class Postprocess:
                 index_list = []
                 doc = None
 
+            # rescale
+            if self.solver.use_scaling:
+                cons_labels = self.model.constraints[name].labels.data
+                scaling_factor = self.optimization_setup.scaling.D_r_inv[cons_labels]
+                arr = arr * scaling_factor
             # create dataframe
             if len(arr.shape) > 0:
                 df = arr.to_series().dropna()
@@ -342,10 +394,25 @@ class Postprocess:
             if len(df.index.names) == len(index_list):
                 df.index.names = index_list
 
+            # rename for operations-only duals
+            if self.optimization_setup.operation_only_phase:
+                name = name + '_operation'
+
             # we transform the dataframe to a json string and load it into the dictionary as dict
             data_frames[name] = self._transform_df(df,doc)
 
-        self.write_file(self.name_dir.joinpath('dual_dict'), data_frames)
+        # choose whether to write new file or append to existing file
+        if self.optimization_setup.operation_only_phase:
+            mode = 'a'
+        else: 
+            mode = 'w'
+
+        # write file
+        self.write_file(
+            self.name_dir.joinpath('dual_dict'), 
+            data_frames, 
+            mode = mode
+        )
 
     def save_system(self):
         """
@@ -365,42 +432,39 @@ class Postprocess:
             fname = self.name_dir.parent.joinpath('analysis')
         else:
             fname = self.name_dir.joinpath('analysis')
+        # remove cwd path part to avoid saving the absolute path
+        if os.path.isabs(self.analysis.dataset):
+            cwd = os.getcwd()
+            self.analysis.dataset = os.path.relpath(self.analysis.dataset,cwd)
+            self.analysis.folder_output = os.path.relpath(self.analysis.folder_output, cwd)
         self.write_file(fname, self.analysis, format="json")
 
-    def save_scenarios(self):
-        """
-        Saves the analysis dict as json
-        """
-
-        # This we only need to save once
-        # check if MF within scenario analysis
-        if isinstance(self.subfolder, tuple):
-            # check if there are sub_scenarios (parent must then be the name of the parent scenario)
-            if not self.subfolder[0].parent == Path("."):
-                fname = self.name_dir.parent.parent.parent.joinpath('scenarios')
-            else:
-                # MF with in scenario analysis without sub-scenarios
-                fname = self.name_dir.parent.parent.joinpath('scenarios')
-        # only MF or only scenario analysis
-        elif self.subfolder != Path(""):
-            fname = self.name_dir.parent.joinpath('scenarios')
-        # neither MF nor scenario analysis
-        else:
-            fname = self.name_dir.joinpath('scenarios')
-        self.write_file(fname, self.scenarios, format="json")
-
     def save_solver(self):
         """
         Saves the solver dict as json
         """
-
         # This we only need to save once
         if self.system.use_rolling_horizon:
             fname = self.name_dir.parent.joinpath('solver')
         else:
             fname = self.name_dir.joinpath('solver')
+        
+        # remove cwd path part to avoid saving the absolute path
+        if os.path.isabs(self.solver.solver_dir):
+            cwd = os.getcwd()
+            self.solver.solver_dir = os.path.relpath(self.solver.solver_dir,cwd)
+        # save    
         self.write_file(fname, self.solver, format="json")
 
+    def save_scenarios(self):
+        """
+        Saves the scenario dict as json
+        """
+        # only save the scenarios at the highest level
+        root_dir = Path(self.analysis.folder_output).joinpath(self.model_name)
+        fname = root_dir.joinpath('scenarios')
+        self.write_file(fname, self.scenarios, format="json")
+
     def save_unit_definitions(self):
         """
         Saves the user-defined units as txt
@@ -442,6 +506,15 @@ class Postprocess:
 
         :param scenario: name of scenario for which results are postprocessed
         """
+        # extract and save sequence time steps, we transform the arrays to lists
+        self.dict_sequence_time_steps = self.flatten_dict(self.energy_system.time_steps.get_sequence_time_steps_dict())
+        self.dict_sequence_time_steps["optimized_time_steps"] = self.optimization_setup.optimized_time_steps
+        self.dict_sequence_time_steps["time_steps_operation_duration"] = self.energy_system.time_steps.time_steps_operation_duration
+        self.dict_sequence_time_steps["time_steps_storage_duration"] = self.energy_system.time_steps.time_steps_storage_duration
+        self.dict_sequence_time_steps["time_steps_storage_level_startend_year"] = self.energy_system.time_steps.time_steps_storage_level_startend_year
+        self.dict_sequence_time_steps["time_steps_year2operation"] = self.get_time_steps_year2operation()
+        self.dict_sequence_time_steps["time_steps_year2storage"] = self.get_time_steps_year2storage()
+
         # add the scenario name
         if scenario is not None:
             add_on = f"_{scenario}"
@@ -466,51 +539,6 @@ class Postprocess:
                 NotImplementedError(f"Type {type(v)} not supported for key {k}")
         self.write_file(fname, dict_formatted, format="json")
 
-    def _transform_df(self, df, doc, units=None):
-        """we transform the dataframe to a json string and load it into the dictionary as dict
-
-        :param df: dataframe
-        :param doc: doc string
-        :param units: units
-        :return: dictionary
-        """
-        if self.output_format == "h5":
-            if units is not None:
-                dataframe = {"dataframe": df, "docstring": doc, "units": units}
-            else:
-                dataframe = {"dataframe": df, "docstring": doc}
-        else:
-            raise AssertionError(f"The specified output format {self.output_format}, chosen in the config, is not supported")
-        return dataframe
-
-    def _doc_to_df(self, doc):
-        """Transforms the docstring to a dataframe
-
-        :param doc: doc string
-        :return: pd.Series of the docstring
-        """
-        if doc is not None:
-            return pd.Series(doc.split(";")).str.split(":",expand=True).set_index(0).squeeze()
-        else:
-            return pd.DataFrame()
-
-    def _unit_df(self, units, index):
-        """Transforms the units to a series
-
-        :param units: units string
-        :param index: index of the target dataframe
-        :return: pd.Series of the units
-        """
-        if units is not None:
-            if isinstance(units, str):
-                return pd.Series(units, index=index)
-            elif len(units) == len(index):
-                units.index.names = index.names
-                return units
-            else:
-                raise AssertionError("The length of the units does not match the length of the index")
-        else:
-            return None
 
     def flatten_dict(self, dictionary):
         """Creates a copy of the dictionary where all numpy arrays are recursively flattened to lists such that it can
@@ -571,3 +599,116 @@ class Postprocess:
         for year, time_steps in self.energy_system.time_steps.time_steps_year2storage.items():
             ans[str(year)] = time_steps
         return ans
+
+    def _transform_df(self, df, doc, units=None):
+        """we transform the dataframe to a json string and load it into the dictionary as dict
+
+        :param df: dataframe
+        :param doc: doc string
+        :param units: units
+        :return: dictionary
+        """
+        if self.output_format == "h5":
+            if units is not None:
+                dataframe = {"dataframe": df, "docstring": doc, "units": units}
+            else:
+                dataframe = {"dataframe": df, "docstring": doc}
+        else:
+            raise AssertionError(f"The specified output format {self.output_format}, chosen in the config, is not supported")
+        return dataframe
+
+    def _doc_to_df(self, doc):
+        """Transforms the docstring to a dataframe
+
+        :param doc: doc string
+        :return: pd.Series of the docstring
+        """
+        if doc is not None:
+            return pd.Series(doc.split(";")).str.split(":",expand=True).set_index(0).squeeze()
+        else:
+            return pd.DataFrame()
+
+    def _unit_df(self, units, index):
+        """Transforms the units to a series
+
+        :param units: units string
+        :param index: index of the target dataframe
+        :return: pd.Series of the units
+        """
+        if units is not None:
+            if isinstance(units, str):
+                return pd.Series(units, index=index)
+            elif len(units) == len(index):
+                units.index.names = index.names
+                return units
+            else:
+                raise AssertionError("The length of the units does not match the length of the index")
+        else:
+            return None
+
+    def _write_h5_file(self, file_name, dictionary, mode = 'w', complevel=4,complib="blosc"):
+        """Writes the dictionary to a hdf5 file
+
+        :param file_name: The name of the file
+        :param dictionary: The dictionary to save
+        :param mode: Writting mode for python file. The two options are 'w' and
+            'a'. The former create a new file while the latter will append to an
+            existing file.
+        """
+        if mode == 'w' and not self.overwrite and os.path.exists(file_name):
+            raise FileExistsError("File already exists. Please set overwrite=True to overwrite the file.")
+        with pd.HDFStore(file_name, mode=mode, complevel=complevel, complib=complib) as store:
+            for key, value in dictionary.items():
+                if not isinstance(key, str):
+                    raise TypeError("All dictionary keys must be strings!")
+                if isinstance(value, dict):
+                    input_dict, docstring, has_units = self._format_dict(value)
+                    if not input_dict["dataframe"].empty:
+                        store.put(key, input_dict["dataframe"], format='table')
+                        # add additional attributes
+                        index_names = input_dict["dataframe"].index.names
+                        index_names = ",".join([str(name) for name in index_names])
+                        store.get_storer(key).attrs.docstring = docstring
+                        store.get_storer(key).attrs["name"] = key
+                        store.get_storer(key).attrs["has_units"] = has_units
+                        store.get_storer(key).attrs["index_names"] = index_names
+                        # remove "_i_table" to reduce file size
+                        try:
+                            store.remove(key + "/_i_table")
+                        except KeyError:
+                            pass
+                else:
+                    raise TypeError(f"Type {type(value)} is not supported.")
+
+    @staticmethod
+    def _format_dict(input_dict):
+        """ format the dictionary to be saved in the hdf file
+        :param input_dict: The dictionary to format
+        """
+        expected_keys = ["dataframe", "docstring"]
+        if "dataframe" in input_dict:
+            df = input_dict["dataframe"]
+            if not isinstance(df, pd.Series):
+                if df.shape[1]:
+                    df = df.squeeze(axis=1)
+            input_dict["dataframe"] = df
+        if "docstring" in input_dict:
+            docstring = input_dict["docstring"]
+        else:
+            docstring = None
+        if "units" in input_dict:
+            units = input_dict["units"]
+            assert isinstance(units, pd.Series), f"Units must be a pandas Series, but is {type(units)}"
+            df = input_dict["dataframe"]
+            assert units.index.intersection(df.index).equals(
+                units.index), f"Units index {units.index} does not match dataframe index {df.index}"
+            units.name = "units"
+            df = pd.concat([df, units], axis=1)
+            input_dict["dataframe"] = df
+            has_units = True
+        else:
+            has_units = False
+        if not (set(input_dict.keys()) == set(expected_keys) or set(input_dict.keys()) == set(expected_keys).union(
+                ["units"])):
+            raise ValueError(f"Expected keys are {expected_keys}, but got {input_dict.keys()}")
+        return input_dict, docstring, has_units
