@@ -1,6 +1,8 @@
 """
 This module contains the implementation of a SolutionLoader that reads the solution.
 """
+import copy
+import warnings
 import re
 import json
 import os
@@ -11,7 +13,7 @@ import numpy as np
 from typing import Optional, Any,Literal
 from enum import Enum
 from functools import cache
-from zen_garden.model.default_config import Analysis, System, Solver
+from zen_garden.default_config import Analysis, System, Solver
 
 class ComponentType(Enum):
     parameter: str = "parameter"
@@ -69,6 +71,7 @@ class Component():
         self,
         name: str,
         component_type: ComponentType,
+        index_names: list[str],
         ts_type: Optional[TimestepType],
         ts_name: Optional[str],
         file_name: str,
@@ -77,6 +80,7 @@ class Component():
     ) -> None:
         self._component_type = component_type
         self._name = name
+        self._index_names = index_names
         self._ts_type = ts_type
         self._file_name = file_name
         self._ts_name = ts_name
@@ -87,6 +91,10 @@ class Component():
     def component_type(self) -> ComponentType:
         return self._component_type
 
+    @property
+    def index_names(self) -> list[str]:
+        return self._index_names
+
     @property
     def timestep_type(self) -> Optional[TimestepType]:
         return self._ts_type
@@ -113,45 +121,149 @@ class Component():
 
 class Scenario():
     """
-    Implementation of the abstract scenario. In this solution version, the analysis and
+    Implementation of the scenario. In this solution version, the analysis and
     system configs are stored as jsons for each of the scenario in the corresponding
     folder.
     """
 
-    def __init__(self, path: str, name: str, base_scenario: str) -> None:
+    def __init__(self, path: str, name: str, base_scenario: str, default_ureg: pint.UnitRegistry) -> None:
+        self.name = name
+        self.base_name = base_scenario
+        self._exists = True
         self._path = path
         self._analysis: Analysis = self._read_analysis()
         self._system: System = self._read_system()
         self._solver: Solver = self._read_solver()
-        self._ureg = self._read_ureg()
-        self.name = name
-        self.base_name = base_scenario
+        self._benchmarking: dict[str,Any] = self._read_benchmarking()
+        self._ureg = self._read_ureg(default_ureg)
+        self._components: dict[str, Component] = self._read_components()
 
     def _read_analysis(self) -> Analysis:
         analysis_path = os.path.join(self.path, "analysis.json")
+        if not os.path.exists(analysis_path):
+            print(f"analysis.json does not exist for scenario {self.name}")
+            self._exists = False
+            return Analysis()
 
         with open(analysis_path, "r") as f:
             return Analysis(**json.load(f))
 
     def _read_system(self) -> System:
         system_path = os.path.join(self.path, "system.json")
+        if not os.path.exists(system_path):
+            print(f"system.json does not exist for scenario {self.name}")
+            return System()
 
         with open(system_path, "r") as f:
             return System(**json.load(f))
 
     def _read_solver(self) -> Solver:
         solver_path = os.path.join(self.path, "solver.json")
+        if not os.path.exists(solver_path):
+            print(f"solver.json does not exist for scenario {self.name}")
+            return Solver()
 
         with open(solver_path, "r") as f:
             return Solver(**json.load(f))
 
-    def _read_ureg(self) -> pint.UnitRegistry:
-        ureg = pint.UnitRegistry()
+    def _read_benchmarking(self) -> dict[str,Any]:
+        benchmarking_path = os.path.join(self.path, "benchmarking.json")
+        if os.path.exists(benchmarking_path):
+            with open(benchmarking_path, "r") as f:
+                return json.load(f)
+        else:
+            return {}
+
+    def _read_ureg(self,default_ureg) -> pint.UnitRegistry:
+        ureg = copy.copy(default_ureg)
         unit_path = os.path.join(self.path, "unit_definitions.txt")
         if os.path.exists(unit_path):
             ureg.load_definitions(unit_path)
         return ureg
 
+    def convert_ts2year(self,df: ["pd.DataFrame","pd.Series"]) -> ["pd.DataFrame","pd.Series"]:
+        """ converts the yearly ts column to the corresponding year """
+        df = df.copy()
+        if isinstance(df, pd.Series):
+            year_index = df.index
+        else:
+            year_index = df.columns
+        assert pd.api.types.is_any_real_numeric_dtype(year_index), f"DataFrame columns must be numeric to convert to year, not {year_index.to_list()}."
+        ry = self.system.reference_year
+        del_y = self.system.interval_between_years
+        years = [ry + i*del_y for i in year_index]
+        if isinstance(df, pd.Series):
+            df.index = years
+        else:
+            df.columns = years
+        return df
+
+    def convert_year2ts(self,year: int) -> int:
+        """ converts the year to the corresponding time step """
+        assert isinstance(year, int), f"Year must be an integer, not {type(year)}."
+        ry = self.system.reference_year
+        del_y = self.system.interval_between_years
+        all_years = [ry + i*del_y for i in range(self.system.optimized_years)]
+        if year in all_years:
+            ts = (year - ry) // del_y
+        elif year <= self.analysis.earliest_year_of_data and year in range(self.system.optimized_years):
+            warnings.warn(f"Selecting the yearly time steps ({year}) instead of the actual year ({ry + del_y*year}) is deprecated. Please use the actual year.", DeprecationWarning)
+            ts = year
+        else:
+            raise KeyError(f"Year {year} not in optimized years {all_years}.")
+        return ts
+
+    def _read_components(self) -> dict[str, Component]:
+        """
+        Create the component instances.
+
+        The components are stored in three files and the file-names define the types of
+        the component. Furthermore, the timestep name and type are derived by checking
+        if any of the defined time steps name is in the index of the dataframe.
+        """
+        ans: dict[str, Component] = {}
+
+        if self.has_rh:
+            mf_name = [i for i in os.listdir(self.path) if "MF_" in i][0]
+            component_folder = os.path.join(self.path, mf_name)
+        else:
+            component_folder = self.path
+
+        for file_name, component_type in ComponentType.get_file_names_maps().items():
+            file_path = os.path.join(component_folder, file_name)
+
+            if not os.path.exists(file_path):
+                continue
+
+            h5_file = h5py.File(file_path)
+            version = get_solution_version(self)
+            for component_name in h5_file.keys():
+                index_names = get_index_names(h5_file,component_name,version)
+                time_index = set(index_names).intersection(set(TimestepType.get_time_steps_names()))
+                timestep_name = time_index.pop() if len(time_index) > 0 else None
+                timestep_type = TimestepType.get_time_step_type(timestep_name)
+
+                doc = get_doc(h5_file,component_name,version)
+
+                has_units = get_has_units(h5_file,component_name,version)
+
+                ans[component_name] = Component(
+                    component_name,
+                    component_type,
+                    index_names,
+                    timestep_type,
+                    timestep_name,
+                    file_name,
+                    doc,
+                    has_units
+                )
+
+        return ans
+
+    @property
+    def components(self) -> dict[str, Component]:
+        return self._components
+
     @property
     def analysis(self) -> Analysis:
         return self._analysis
@@ -160,6 +272,14 @@ class Scenario():
     def solver(self) -> Solver:
         return self._solver
 
+    @property
+    def system(self) -> System:
+        return self._system
+
+    @property
+    def benchmarking(self) -> dict[str,Any]:
+        return self._benchmarking
+
     @property
     def path(self) -> str:
         return self._path
@@ -168,14 +288,14 @@ class Scenario():
     def has_rh(self) -> bool:
         return self.system.use_rolling_horizon
 
-    @property
-    def system(self) -> System:
-        return self._system
-
     @property
     def ureg(self) -> pint.UnitRegistry:
         return self._ureg
 
+    @property
+    def exists(self) -> bool:
+        return self._exists
+
 class SolutionLoader():
     """
     Implementation of a SolutionLoader.
@@ -185,17 +305,12 @@ class SolutionLoader():
         self.path = path
         assert len(os.listdir(path)) > 0, f"Path {path} is empty."
         self._scenarios: dict[str, Scenario] = self._read_scenarios()
-        self._components: dict[str, Component] = self._read_components()
         self._series_cache: dict[str, "pd.Series[Any]"] = {}
 
     @property
     def scenarios(self) -> dict[str, Scenario]:
         return self._scenarios
 
-    @property
-    def components(self) -> dict[str, Component]:
-        return self._components
-
     @property
     def name(self) -> str:
         scenario = get_first_scenario(self._scenarios)
@@ -277,18 +392,22 @@ class SolutionLoader():
         series.index.names = new_index_names
         return series
 
+    @cache
     def get_component_data(
         self,
         scenario: Scenario,
         component: Component,
         keep_raw: bool = False,
-        data_type: Literal["dataframe","units"] = "dataframe"
+        data_type: Literal["dataframe","units"] = "dataframe",
+        index = None
     ) -> "pd.DataFrame | pd.Series[Any]":
         """
         Returns the actual component values given
         a component and a scenario. Already combines the yearly data if the solution does
         not use perfect foresight, unless explicitly desired otherwise (keep_raw = True).
         """
+        if index is None:
+            index = tuple()
         version = get_solution_version(scenario)
         if scenario.has_rh:
             # If solution has rolling horizon, load the values for all the foresight
@@ -310,7 +429,7 @@ class SolutionLoader():
                     scenario.path, subfolder_name, component.file_name
                 )
                 pd_series_dict[mf_idx] = get_df_from_path(
-                    file_path, component.name,version, data_type
+                    file_path, component.name,version, data_type, index
                 )
             if not keep_raw:
                 combined_dataseries = self._combine_dataseries(
@@ -324,7 +443,7 @@ class SolutionLoader():
         else:
             # If solution does not use rolling horizon, simply load the HDF file.
             file_path = os.path.join(scenario.path, component.file_name)
-            ans = get_df_from_path(file_path, component.name,version,data_type)
+            ans = get_df_from_path(file_path, component.name,version,data_type, index)
             return ans
 
     def _read_scenarios(self) -> dict[str, Scenario]:
@@ -335,14 +454,14 @@ class SolutionLoader():
         """
         scenarios_json_path = os.path.join(self.path, "scenarios.json")
         ans: dict[str, Scenario] = {}
-
+        default_ureg = pint.UnitRegistry()
         with open(scenarios_json_path, "r") as f:
             scenario_configs = json.load(f)
 
         if len(scenario_configs) == 1:
             scenario_name = "none"
             scenario_path = self.path
-            ans[scenario_name] = Scenario(scenario_path, scenario_name, "")
+            ans[scenario_name] = Scenario(scenario_path, scenario_name, "",default_ureg)
         else:
             for scenario_id, scenario_config in scenario_configs.items():
                 scenario_name = f"scenario_{scenario_id}"
@@ -361,56 +480,12 @@ class SolutionLoader():
                         scenario_path, f"scenario_{scenario_subfolder}"
                     )
 
-                ans[scenario_name] = Scenario(
-                    scenario_path, scenario_name, base_scenario
+                scenario = Scenario(
+                    scenario_path, scenario_name, base_scenario, default_ureg
                 )
 
-        return ans
-
-    def _read_components(self) -> dict[str, Component]:
-        """
-        Create the component instances.
-
-        The components are stored in three files and the file-names define the types of
-        the component. Furthermore, the timestep name and type are derived by checking
-        if any of the defined time steps name is in the index of the dataframe.
-        """
-        ans: dict[str, Component] = {}
-        first_scenario = get_first_scenario(self.scenarios)
-
-        if first_scenario.has_rh:
-            mf_name = [i for i in os.listdir(first_scenario.path) if "MF_" in i][0]
-            component_folder = os.path.join(first_scenario.path, mf_name)
-        else:
-            component_folder = first_scenario.path
-
-        for file_name, component_type in ComponentType.get_file_names_maps().items():
-            file_path = os.path.join(component_folder, file_name)
-
-            if not os.path.exists(file_path):
-                continue
-
-            h5_file = h5py.File(file_path)
-            version = get_solution_version(first_scenario)
-            for component_name in h5_file.keys():
-                index_names = get_index_names(h5_file,component_name,version)
-                time_index = set(index_names).intersection(set(TimestepType.get_time_steps_names()))
-                timestep_name = time_index.pop() if len(time_index) > 0 else None
-                timestep_type = TimestepType.get_time_step_type(timestep_name)
-
-                doc = get_doc(h5_file,component_name,version)
-
-                has_units = get_has_units(h5_file,component_name,version)
-
-                ans[component_name] = Component(
-                    component_name,
-                    component_type,
-                    timestep_type,
-                    timestep_name,
-                    file_name,
-                    doc,
-                    has_units
-                )
+                if scenario.exists:
+                    ans[scenario_name] = scenario
 
         return ans
 
@@ -429,7 +504,7 @@ class SolutionLoader():
         version = get_solution_version(scenario)
         if check_if_v1_leq_v2(version,"v0"):
             time_step_duration = self.get_component_data(
-                scenario, self.components[timestep_duration_name]
+                scenario, scenario.components[timestep_duration_name]
             )
         else:
             time_steps_file_name = _get_time_steps_file(scenario)
@@ -501,6 +576,8 @@ class SolutionLoader():
             time_step_name = "time_steps_year2storage"
         elif ts_type is TimestepType.operational:
             time_step_name = "time_steps_year2operation"
+        else:
+            raise KeyError(f"Time step type {ts_type} not found.")
 
         time_step_yearly = time_step_file[time_step_name]
 
@@ -600,9 +677,6 @@ class SolutionLoader():
         return dict_startend
 
 #### Helper functions
-
-
-
 def get_first_scenario(scenarios: dict[str, Scenario]) -> Scenario:
     """
     Helper-function that returns the first scenario of a dictionary of scenarios.
@@ -621,7 +695,7 @@ def get_solution_version(scenario: Scenario) -> str:
 
     :return: The version of the solution.
     """
-    versions = {"v1":"2.0.14","v2":"2.2.9"}
+    versions = {"v1":"2.0.14","v2":"2.2.15"}
     version = "v0"
     if hasattr(scenario.analysis,"zen_garden_version"):
         for k,v in versions.items():
@@ -704,20 +778,29 @@ def get_has_units(h5_file: h5py.File,component_name: str,version: str) -> bool:
     return has_units
 
 @cache
-def get_df_from_path(path: str, component_name: str, version: str, data_type: Literal["dataframe","units"] = "dataframe") -> "pd.Series[Any]":
+def get_df_from_path(path: str, component_name: str, version: str, data_type: Literal["dataframe","units"] = "dataframe",index: Optional[tuple[str]] = None) -> "pd.Series[Any]":
     """
     Helper-function that returns a Pandas series given the path of a file and the
     component name.
     """
+    if index is None:
+        index = tuple()
+
     if check_if_v1_leq_v2(version,"v0"):
         pd_read = pd.read_hdf(path, component_name + f"/{data_type}")
     else:
         if data_type == "dataframe":
-            pd_read = pd.read_hdf(path, component_name)
+            try:
+                pd_read = pd.read_hdf(path, component_name,where=index)
+            except:
+                pd_read = pd.read_hdf(path, component_name)
             if isinstance(pd_read, pd.DataFrame):
                 pd_read = pd_read["value"]
         elif data_type == "units":
-            pd_read = pd.read_hdf(path, component_name)["units"]
+            try:
+                pd_read = pd.read_hdf(path, component_name,where=index)["units"]
+            except:
+                pd_read = pd.read_hdf(path, component_name)["units"]
         else:
             raise ValueError(f"Data type {data_type} not supported.")
 
@@ -725,8 +808,10 @@ def get_df_from_path(path: str, component_name: str, version: str, data_type: Li
         ans = pd_read.squeeze()
     elif isinstance(pd_read, pd.Series):
         ans = pd_read
+    else:
+        raise ValueError(f"Data type {type(pd_read)} not supported.")
 
-    if isinstance(ans, (np.float_, str)):
+    if isinstance(ans, (np.float64, str)):
         ans = pd.Series([ans], index=pd_read.index)
 
     assert type(ans) is pd.Series
@@ -740,11 +825,8 @@ def _get_time_steps_file(scenario):
     :param scenario:
     :return: time_steps_file_name
     """
-    time_steps_file_name = [
-        i.split(".")[0]
-        for i in os.listdir(scenario.path)
-        if "dict_all_sequence_time_steps" in i and ".lock" not in i
-    ]
+    time_steps_file_name = [os.path.splitext(i)[0] for i in os.listdir(scenario.path)
+                            if 'dict_all_sequence_time_steps' in i and '.lock' not in i]
     time_steps_file_name = np.unique(time_steps_file_name)
     assert len(time_steps_file_name) == 1, f"Multiple time steps files found: {time_steps_file_name}"
     time_steps_file_name = time_steps_file_name[0]
