@@ -16,7 +16,7 @@ import xarray as xr
 import shutil
 from copy import deepcopy
 from pathlib import Path
-from zen_garden.model.default_config import Subscriptable
+from zen_garden.default_config import Subscriptable
 
 def setup_logger(level=logging.INFO):
     """ set up logger
@@ -225,12 +225,12 @@ def reformat_slicing_index(index, component) -> tuple[str]:
     index_names = component.index_names
     if isinstance(index, str) or isinstance(index, float) or isinstance(index, int):
         index_name = index_names[0]
-        ref_index = (f"{index_name} == {index}",)
+        ref_index = (f"'{index_name}' == '{index}'",)
         if len(index_names) == 1:
-            ref_index = (f"index == {index}",)
+            ref_index = (f"index == '{index}'",)
     elif isinstance(index, list):
         index_name = index_names[0]
-        ref_index = (f"{index_name} in {index}",)
+        ref_index = (f"'{index_name}' in {index}",)
     elif isinstance(index, dict):
         ref_index = []
         for key, value in index.items():
@@ -238,9 +238,9 @@ def reformat_slicing_index(index, component) -> tuple[str]:
                 warnings.warn(f"Invalid index name '{key}' in index. Skipping.", Warning)
                 continue
             if isinstance(value, list):
-                ref_index.append(f"{key} in {value}")
+                ref_index.append(f"'{key}' in {value}")
             else:
-                ref_index.append(f"{key} == {value}")
+                ref_index.append(f"'{key}' == '{value}'")
         ref_index = tuple(ref_index)
     elif isinstance(index, tuple):
         ref_index = []
@@ -252,9 +252,9 @@ def reformat_slicing_index(index, component) -> tuple[str]:
             if index[i] is None:
                 continue
             elif isinstance(index[i], list):
-                ref_index.append(f"{index_name} in {index[i]}")
+                ref_index.append(f"'{index_name}' in {index[i]}")
             else:
-                ref_index.append(f"{index_name} == {index[i]}")
+                ref_index.append(f"'{index_name}' == '{index[i]}'")
         ref_index = tuple(ref_index)
     else:
         warnings.warn(f"Invalid index type {type(index)}. Skipping.", Warning)
@@ -828,7 +828,7 @@ class InputDataChecks:
         Checks selection of different technologies in system.py file
         """
         # Checks if at least one technology is selected in the system.py file
-        assert len(self.system.set_conversion_technologies + self.system.set_transport_technologies + self.system.set_storage_technologies) > 0, f"No technology selected in system.py"
+        assert len(self.system.set_conversion_technologies + self.system.set_transport_technologies + self.system.set_storage_technologies) > 0, f"No technology selected in system"
         # Checks if identical technologies are selected multiple times in system.py file and removes possible duplicates
         for tech_list in ["set_conversion_technologies", "set_transport_technologies", "set_storage_technologies"]:
             techs_selected = getattr(self.system,tech_list)
